#!/usr/bin/env python

import os
from optparse import OptionParser
from subprocess import Popen, PIPE


def main():

    usage = "usage: %prog [options] FILE MD5SUM"
    parser = OptionParser(usage)

    (options, args) = parser.parse_args()

    if len(args) != 2:
        raise Exception('Number Of Args != 2')
    cvmfs_test_file = args[0]
    cvmfs_md5sum = args[1]

    try:
        if os.path.isfile(cvmfs_test_file):
            cmd = ['md5sum', cvmfs_test_file]
            p = Popen(cmd, stdout=PIPE)
            output = p.communicate()[0]
            if cvmfs_md5sum not in output:
                raise Exception()
        else:
            raise Exception()
        # If we got this far CVMFS must work
        print 'PYGLIDEIN_RESOURCE_CVMFS=True'
        print '- update:true'
    except:
        print 'PYGLIDEIN_RESOURCE_CVMFS=False'
        print '- update:true'


if __name__ == '__main__':
    main()



import logging
import pickle
import socket
import struct
import sys


class MetricsSenderClient(object):

    client_metrics_namespace = {
        'glideins_launched': 'glideins.launched',
        'glideins_running': 'glideins.running',
        'glideins_idle': 'glideins.idle',
        'avg_idle_time': 'glideins.avg_idle_time',
        'min_idle_time': 'glideins.min_idle_time',
        'max_idle_time': 'glideins.max_idle_time'
    }

    def __init__(self, config):

        self.logger = logging.getLogger('server')
        self.graphite_server = config.get('graphite_server', None)
        if self.graphite_server is None:
            self.logger.error('graphite_server not defined in configuration.')
            sys.exit(1)
        self.graphite_port = config.get('graphite_port', 2004)
        self.namespace = config.get('namespace', 'pyglidein')

    def send(self, metrics_bundle):

            payload = []
            uuid = metrics_bundle.get_uuid()
            timestamp = metrics_bundle.get_timestamp()
            metrics = metrics_bundle.get_metrics()
            for metric in metrics:
                for partition in metrics[metric]:
                    path = str('.'.join([self.namespace, uuid, partition,
                                        MetricsSenderClient.client_metrics_namespace[metric]]))
                    payload.append((path, (timestamp, metrics[metric][partition])))
            self.logger.debug(payload)
            payload = pickle.dumps(payload, protocol=2)
            header = struct.pack("!L", len(payload))
            message = header + payload

            sock = socket.socket()
            sock.connect((self.graphite_server, self.graphite_port))
            sock.sendall(message)
            sock.close()



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function

import subprocess
import threading
import logging
from functools import partial
from optparse import OptionParser
from collections import Counter
import distutils.version
from datetime import datetime
import re

from pyglidein.util import json_encode, json_decode
from pyglidein.config import Config
from pyglidein.metrics_sender_client import MetricsSenderClient
from pyglidein.client_metrics import ClientMetricsBundle
import tornado.escape
tornado.escape.json_encode = json_encode
tornado.escape.json_decode = json_decode

from tornado.ioloop import IOLoop
from tornado.httpserver import HTTPServer
import tornado.web
import tornado.gen

logger = logging.getLogger('server')

job_status = {
    1:'Idle',
    2:'Run',
    3:'Del',
    4:'OK',
    5:'Held',
    6:'Err',
}
def get_job_status(st):
    return job_status[st] if st in job_status else 'Unk'

job_universe = {
    1:'standard',
    4:'PVM',
    5:'vanilla',
    6:'PVMd',
    7:'scheduler',
    8:'MPI',
    9:'grid',
    10:'java',
    11:'parallel',
    12:'local',
}

client_metrics = {
    'glideins_launched': 'glideins.launched',
    'glideins_running': 'glideins.running',
    'glideins_idle': 'glideins.idle',
    'avg_idle_time': 'glideins.avg_idle_time',
    'min_idle_time': 'glideins.min_idle_time',
    'max_idle_time': 'glideins.max_idle_time'
}

class MyHandler(tornado.web.RequestHandler):
    """Default Handler"""
    def initialize(self, cfg):
        """
        Get some params from the website module

        Args:
            cfg: the global config
        """
        self.cfg = cfg
        if self.cfg['metrics_sender_client'] is not None:
            self.metrics_sender_client = self.cfg['metrics_sender_client']
        else:
            self.metrics_sender_client = None

    def get(self):
        """GET is invalid and returns an error"""
        raise tornado.web.HTTPError(400, 'GET is invalid.  Use POST')

    def post(self):
        """POST is invalid and returns an error"""
        raise tornado.web.HTTPError(400, 'POST is invalid.  Use GET')

class JSONRPCHandler(MyHandler):
    """
    JSONRPC 2.0 Handler.

    Call DB methods using RPC over json.
    """

    def post(self):
        """Parses json in the jsonrpc format, returning results in
           jsonrpc format as well.
        """
        # parse JSON
        try:
            request = tornado.escape.json_decode(self.request.body)
        except Exception as e:
            raise tornado.web.HTTPError(400, 'POST request is not valid json')

        # check for all parts of jsonrpc 2.0 spec
        if 'jsonrpc' not in request or request['jsonrpc'] not in ('2.0', 2.0):
            self.json_error({'code':-32600, 'message':'Invalid Request',
                             'data':'jsonrpc is not 2.0'})
        elif 'method' not in request:
            self.json_error({'code':-32600, 'message':'Invalid Request',
                             'data':'method not in request'})
        elif request['method'].startswith('_'):
            self.json_error({'code':-32600, 'message':'Invalid Request',
                             'data':'method name cannot start with underscore'})
        else:
            method = request['method']
            if 'params' in request:
                params = request['params']
            else:
                params = {}
            if 'id' in request:
                request_id = request['id']
            else:
                request_id = None

            # call method
            try:
                if method == 'get_state':
                    ret = self.cfg['state']
                elif method == 'monitoring':
                    client_id = params.pop('uuid')
                    client_id_clean = re.sub(r'\W+', '', client_id)
                    # For Clients > 1.1
                    if 'timestamp' in params and 'metrics' in params:
                        metrics_bundle = ClientMetricsBundle(client_id_clean,
                                                             timestamp=params['timestamp'],
                                                             metrics=params['metrics'])
                    # For Clients < 1.1
                    else:
                        metrics_bundle = ClientMetricsBundle(client_id_clean,
                                                             metrics=params)
                    if self.metrics_sender_client is not None:
                        self.metrics_sender_client.send(metrics_bundle)
                    # Continue sending metrics to old web interface
                    self.cfg['monitoring'][client_id] = metrics_bundle.get_v1_bundle()
                    ret = ''
                else:
                    self.json_error({'code':-32601, 'message':'Method not found'},
                                    request_id=request_id)
                    return
            except:
                self.json_error({'code':-32602, 'message':'Invalid params',
                                 'data':str(ret)}, request_id=request_id)
            else:
                # return response
                self.write({'jsonrpc':'2.0', 'result':ret, 'id':request_id})

    def json_error(self, error, status=400, request_id=None):
        """Create a proper jsonrpc error message"""
        self.set_status(status)
        if isinstance(error, Exception):
            error = str(error)
        logger.info('json_error: %r', error)
        self.write({'jsonrpc':'2.0', 'error':error, 'id':request_id})


class DefaultHandler(MyHandler):
    """Display queue status in html"""
    def get(self):
        self.write("""
<html>
<head>
  <title>Queue Status</title>
  <style>
    .num {
      margin-left: 1em;
    }
    div.clients>div, div.reqs>div {
      margin: .2em;
    }
    div.clients {
      margin-bottom: 1em;
    }
    div.clients>div>span, div.reqs>div>span {
      margin-right: .5em;
      width: 5em;
      display: inline-block;
      vertical-align: top;
    }
    div.clients>div>span {
      width: 15em;
      word-wrap: break-word;
    }
  </style>
</head>
<body>
  <h1>Pyglidein Server</h1>
  <h2>List of requirements</h2>
  <div class="reqs">
    <div><span class="num">Num</span><span>CPUs</span><span>Memory</span><span>Disk</span><span>GPUs</span><span>OS</span></div>""")
        for row in self.cfg['state']:
            self.write('<div><span class="num">'+str(row['count'])+'</span><span>'+str(row['cpus'])+'</span><span>'+str(row['memory'])+'</span><span>'+str(row['disk'])+'</span><span>'+str(row['gpus'])+'</span><span>'+str(row['os'])+'</span></div>')
        self.write("""
  </div>
  <h2>Clients</h2>
  <div class="clients">
    <div><span>UUID</span><span>Last update</span><span>Stats</span></div>""")
        for uuid in self.cfg['monitoring']:
            try:
                info = self.cfg['monitoring'][uuid]
                timestamp = datetime.fromtimestamp(info['timestamp']).strftime('%Y-%m-%d %H:%M:%S')
                stats = '<br>'.join(str(k)+': '+str(info[k]) for k in info if k != 'timestamp')
                self.write('<div><span class="uuid">'+str(uuid)+'</span><span class="date">'+timestamp+'</span><span class="stats">'+stats+'</span></div>')
            except Exception:
                logging.info('error in monitoring display: %r %r',uuid,self.cfg['monitoring'][uuid],exc_info=True)
                continue
        self.write("""
  </div>
</body>
</html>""")


class server:
    def __init__(self, cfg):
        self.cfg = cfg
        handler_args = {'cfg':self.cfg}
        self.application = tornado.web.Application([
            (r"/jsonrpc", JSONRPCHandler, handler_args),
            (r"/.*", DefaultHandler, handler_args),
        ])
    def start(self):
        self.http_server = HTTPServer(self.application, xheaders=True)
        self.http_server.listen(self.cfg["options"].port)
        IOLoop.instance().start()
    def stop(self):
        self.http_server.stop()
        IOLoop.instance().stop()

def get_condor_version():
     p = subprocess.Popen("condor_version", shell=True, stdout=subprocess.PIPE)
     out = p.communicate()[0]
     return out.split(" ")[1]

def condor_q(cfg):
    """Get the status of the HTCondor queue"""
    logger.info('condor_q')
    cmd = ['condor_q', '-global', '-autoformat:,', 'RequestCPUs', 'RequestMemory',
           'RequestDisk', 'RequestGPUs', '-format', '"%s"', 'Requirements',
           '-constraint', '"JobStatus =?= 1"']
    if cfg['options'].constraint:
        cmd += ['-constraint', cfg['options'].constraint]
    if cfg['options'].user:
        cmd += [cfg['options'].user]
    if (distutils.version.LooseVersion(get_condor_version()) >=
         distutils.version.LooseVersion("8.5.2") and
         not cfg['options'].user):
        cmd += ["-allusers"]

    state = []
    try:
        cmd = ' '.join(cmd)
        print(cmd)
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        output = p.communicate()[0]
        for line in output.splitlines():
            logger.debug(line)
            try:
                cpus, memory, disk, gpus, reqs = line.split(', ',4)
                cpus = 1 if cpus == 'undefined' else int(cpus)
                memory = 2000 if memory == 'undefined' else int(memory)
                disk = 10000 if disk == 'undefined' else int(disk)/1000 # convert to MB
                gpus = 0 if gpus == 'undefined' else int(gpus)
                req_os = None
                if 'OpSysAndVer =?= "SL6"' in reqs:
                    req_os = 'sl6'
                state.append((cpus, memory, disk, gpus, req_os))
            except Exception:
                logger.info('error parsing line', exc_info=True)
                continue
        state = [{'cpus':s[0], 'memory':s[1], 'disk':s[2],
                  'gpus':s[3], 'os':s[4], 'count': count} 
                  for s, count in Counter(state).items()]
    except Exception:
        logger.warn('error in condor_q', exc_info=True)
        state = None
    finally:
        def cb():
            # update state only on the main io loop
            logger.info('state is updated to %r', state)
            if state is not None:
                cfg['state'] = state
            cfg['condor_q'] = False
            IOLoop.instance().call_later(cfg['options'].delay,
                                         partial(condor_q_helper, cfg))
        IOLoop.instance().add_callback(cb)

def condor_q_helper(cfg):
    """Helper to launch condor_q in a separate thread"""
    # make sure we're not already running a condor_q
    if not cfg['condor_q']:
        cfg['condor_q'] = True
        t = threading.Thread(target=partial(condor_q, cfg))
        t.daemon = True
        t.start()


def main():
    parser = OptionParser()
    parser.add_option('-p', '--port', type='int', default=11001,
                      help='Port to serve from (default: 11001)')
    parser.add_option('-u', '--user', type='string', default=None,
                      help='Only track a single user')
    parser.add_option('--constraint', type='string', default=None,
                      help='HTCondor constraint expression')
    parser.add_option('--delay', type='int', default=300,
                      help='delay between calls to condor_q (default: 300 seconds)')
    parser.add_option('--debug', action='store_true', default=False,
                      help='Enable debug logging')
    parser.add_option('--config', type='string', default='pyglidein_server.config',
                      help="config file for cluster")
    (options, args) = parser.parse_args()

    config = Config(options.config)

    if options.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    if options.delay < 0 or options.delay > 1000:
        raise Exception('delay out of range')
        
    if config.get('metrics', {}).get('enable_metrics', False):
        metrics_sender_client = MetricsSenderClient(config['metrics'])
    else: metrics_sender_client = None

    cfg = {'options': options, 'config': config, 'condor_q': False, 'state': [], 'monitoring': {},
           'metrics_sender_client': metrics_sender_client}

    # load condor_q
    IOLoop.instance().call_later(5, partial(condor_q_helper, cfg))

    # setup server
    s = server(cfg)
    s.start()

if __name__ == '__main__':
    main()



"""Utilities used by client.py and ssh_helper.py"""
from __future__ import absolute_import, division, print_function

import logging
import threading
import urllib2
import ast
import datetime

from pyglidein.util import json_encode, json_decode

logger = logging.getLogger('client_util')

class Client(object):
    """Raw JSONRPC client object"""
    cid = 0
    cidlock = threading.RLock()

    def __init__(self, timeout=60.0, address=None, ssl_options=None):
        if address is None:
            raise Exception('need a valid address')
        # save timeout
        self._timeout = timeout
        # save address
        self._address = address
        # save ssl_options
        self._sslopts = ssl_options

    @classmethod
    def newid(cls):
        cls.cidlock.acquire()
        cid = cls.cid
        cls.cid += 1
        cls.cidlock.release()
        return cid

    def request(self, methodname, kwargs):
        """Send request to RPC Server"""
        # check method name for bad characters
        if methodname[0] == '_':
            logger.warning('cannot use RPC for private methods')
            raise Exception('Cannot use RPC for private methods')

        # translate request to json
        body = json_encode({'jsonrpc': '2.0', 'method': methodname,
                            'params': kwargs, 'id': Client.newid()})

        headers = {'Content-type':'application/json'}
        request = urllib2.Request(self._address, data=body, headers=headers)

        # make request to server
        try:
            response = urllib2.urlopen(request, timeout=self._timeout)
        except Exception:
            logger.warn('error making jsonrpc request', exc_info=True)
            raise

        # translate response from json
        try:
            cb_data = response.read()
            data = json_decode(cb_data)
        except Exception:
            try:
                logger.info('json data: %r', cb_data)
            except Exception:
                pass
            raise

        if 'error' in data:
            try:
                raise Exception('Error %r: %r    %r'%data['error'])
            except Exception:
                raise Exception('Error %r'%data['error'])
        if 'result' in data:
            return data['result']
        else:
            return None

def get_state(address):
    """Getting the server state directly from remote queue"""
    c = Client(address=address)
    try:
        return c.request('get_state', {})
    except Exception:
        logger.warn('error getting state', exc_info=True)

def monitoring(address,info=None):
    """Sending monitoring information back"""
    if info is None:
        info = {}
    c = Client(address=address)
    try:
        return c.request('monitoring', info)
    except Exception:
        logger.warn('error getting state', exc_info=True)

def config_options_dict(config):
    """
    Parsing config file

    Args:
        config: Pythong config parser object

    Returns:
        A dict with the different sections of the config file
        and the literal values of the configuraton objects
    """
    config_dict = {}
    for section in config.sections():
        config_dict[section] = {}
        for option in config.options(section):
            val = config.get(section, option)
            try:
                val = ast.literal_eval(val)
            except Exception:
                pass
            config_dict[section][option] = val
    return config_dict


def get_presigned_put_url(filename, config, secrets):
    """Generate a presigned put URL using the Minio S3 client.

    Args:
        filename: Name of file to use in S3
        config: Pyglidein cluster config dictionary
        secrets: Pyglidein cluster secrets dictionary

    Returns:
        string: Presigned Put URL

    """
    from minio import Minio
    from minio.error import ResponseError

    config_startd_logging = config['StartdLogging']
    secrets_startd_logging = secrets['StartdLogging']

    client = Minio(config_startd_logging['url'],
                   access_key=secrets_startd_logging['access_key'],
                   secret_key=secrets_startd_logging['secret_key'],
                   secure=True
                   )

    try:
        return client.presigned_put_object(config_startd_logging['bucket'],
                                           filename,
                                           datetime.timedelta(days=1))
    except ResponseError as err:
        print(err)


def get_presigned_get_url(filename, config, secrets):
    """Generate a presigned get URL using the Minio S3 client.

    Args:
        filename: Name of file to use in S3
        config: Pyglidein cluster config dictionary
        secrets: Pyglidein cluster secrets dictionary

    Returns:
        string: Presigned Get URL

    """
    from minio import Minio
    from minio.error import ResponseError
    
    config_startd_logging = config['StartdLogging']
    secrets_startd_logging = secrets['StartdLogging']

    client = Minio(config_startd_logging['url'],
                   access_key=secrets_startd_logging['access_key'],
                   secret_key=secrets_startd_logging['secret_key'],
                   secure=True
                   )

    try:
        return client.presigned_get_object(config_startd_logging['bucket'],
                                           filename)
    except ResponseError as err:
        print(err)



"""
Some JSON encoding and decoding utilities.
"""

from __future__ import absolute_import, division, print_function

import json
from datetime import date, datetime, time
import logging
from collections import defaultdict

logger = logging.getLogger('util')

class datetime_converter:
    @staticmethod
    def dumps(obj):
        return obj.isoformat()
    @staticmethod
    def loads(obj, name=None):
        if ':' in obj:
            if 'T' in obj or ' ' in obj:
                center = ' '
                if 'T' in obj:
                    center = 'T'
                # must be datetime
                if '.' in obj:
                    return datetime.strptime(obj, "%Y-%m-%d"+center+"%H:%M:%S.%f")
                else:
                    return datetime.strptime(obj, "%Y-%m-%d"+center+"%H:%M:%S")
            else:
                # must be time
                if '.' in obj:
                    return datetime.strptime(obj, "%H:%M:%S.%f")
                else:
                    return datetime.strptime(obj, "%H:%M:%S")
        else:
            # must be date
            return datetime.strptime(obj, "%Y-%m-%d")

class date_converter(datetime_converter):
    @staticmethod
    def loads(obj, name=None):
        d = datetime_converter.loads(obj)
        return date(d.year, d.month, d.day)

class time_converter(datetime_converter):
    @staticmethod
    def loads(obj, name=None):
        d = datetime_converter.loads(obj)
        return time(d.hour, d.minute, d.second, d.microsecond)

JSONConverters = {
    'datetime':datetime_converter,
    'date':date_converter,
    'time':time_converter,
}

def objToJSON(obj):
    if isinstance(obj,(dict,list,tuple,str,unicode,int,long,float,bool)) or obj is None:
        return obj
    else:
        name = obj.__class__.__name__
        if name in JSONConverters:
            return {'__jsonclass__': [name, JSONConverters[name].dumps(obj)]}
        else:
            raise Exception('Cannot encode %s class to JSON'%name)

def JSONToObj(obj):
    logger.debug(obj)
    ret = obj
    if isinstance(obj, dict) and '__jsonclass__' in obj:
        logger.info('try unpacking class')
        try:
            name = obj['__jsonclass__'][0]
            if name not in JSONConverters:
                raise Exception('class %r not found in converters'%name)
            obj_repr = obj['__jsonclass__'][1]
            ret = JSONConverters[name].loads(obj_repr, name=name)
        except Exception as e:
            logger.warn('error making json class', exc_info=True)
    return ret


# copied from tornado.escape so we don't have to include that project
def recursive_unicode(obj):
    """Walks a simple data structure, converting byte strings to unicode.

    Supports lists, tuples, and dictionaries.
    """
    if isinstance(obj, dict):
        return dict((recursive_unicode(k), recursive_unicode(v)) for (k, v) in obj.iteritems())
    elif isinstance(obj, list):
        return list(recursive_unicode(i) for i in obj)
    elif isinstance(obj, tuple):
        return tuple(recursive_unicode(i) for i in obj)
    elif isinstance(obj, bytes):
        return obj.decode("utf-8")
    else:
        return obj


def json_encode(value):
    """JSON-encodes the given Python object."""
    return json.dumps(recursive_unicode(value), default=objToJSON,
                      separators=(',', ':')).replace('</', '<\\/')

def json_decode(value):
    """Returns Python objects for the given JSON string."""
    return json.loads(value, object_hook=JSONToObj)
    

def counter(states):
    out_dict = defaultdict(int)
    for s in states:
        out_dict[s] += 1
    return out_dict



#!/usr/bin/env python

import os
from optparse import OptionParser
from subprocess import check_output, STDOUT
import xml.etree.ElementTree as ET

OUTPUT_FILE = 'benchmark.xml'

CLASSAD_MAP = {'I3CLSimModule_makeCLSimHits_makePhotons_clsim_AverageDeviceTimePerPhoton':
               'PYGLIDEIN_METRIC_TIME_PER_PHOTON'}


def main():

    usage = "usage: %prog [options]"
    parser = OptionParser(usage)
    parser.add_option('-n', type='str', default=1,
                      help="Number of simulations to run")
    (options, args) = parser.parse_args()

    try:
        env = os.environ
        gpus = env.get('GPUS', '0')
        if gpus == '0':
            print 'PYGLIDEIN_RESOURCE_GPU=False'
            print '- update:true'
        else:
            gpu = gpus.split(',')[0][4]
            env['CUDA_VISIBLE_DEVICES'] = gpu
            cmd = []
            cmd.append(os.path.join('/cvmfs/icecube.opensciencegrid.org/py2-v2',
                                    os.environ['OS_ARCH'],
                                    'metaprojects/simulation/V05-00-07/env-shell.sh'))
            cmd.append(os.path.join('/cvmfs/icecube.opensciencegrid.org/py2-v2',
                                    os.environ['OS_ARCH'],
                                    'metaprojects/simulation/V05-00-07/clsim/resources',
                                    'scripts/benchmark.py'))
            cmd.extend(['-n', options.n])

            check_output(cmd, shell=False, env=env, stderr=STDOUT)

            tree = ET.parse(OUTPUT_FILE)
            root = tree.getroot()
            for child in root[0][1]:
                if child.tag == 'item' and child[0].text in CLASSAD_MAP:
                        print '{}={}'.format(CLASSAD_MAP[child[0].text], child[1].text)

            print 'PYGLIDEIN_RESOURCE_GPU=True'
            print '- update:true'
    except:
        print 'PYGLIDEIN_RESOURCE_GPU=False'
        print '- update:true'


if __name__ == '__main__':
    main()



"""
Create a glidein tarball by downloading the source, building it, then
copying what is needed into the tarball.
"""

import sys
import os
import stat
import shutil
import subprocess
import tarfile
import tempfile

if sys.version_info[0] < 3 and sys.version_info[1] < 7:
    raise Exception('requires python 2.7+')

def libuuid_download(version='1.0.3'):
    url = 'http://downloads.sourceforge.net/project/libuuid/libuuid-'+version+'.tar.gz'
    subprocess.check_call(['wget', url])
    subprocess.check_call(['tar', '-zxf', 'libuuid-'+version+'.tar.gz'])
    return 'libuuid-'+version

def libuuid_build():
    """Build uuid statically"""
    dirname = libuuid_download()
    initial_dir = os.getcwd()
    os.chdir(dirname)
    try:
        if os.path.exists('release_dir'):
            shutil.rmtree('release_dir')
        os.mkdir('release_dir')
        options = ['--enable-static',
                   '--disable-shared',
                   '--prefix',os.path.join(initial_dir,dirname,'release_dir'),
                  ]
        subprocess.check_call(['./configure']+options)
        subprocess.check_call(['make'])
        subprocess.check_call(['make','install'])
        return os.path.join(initial_dir,dirname,'release_dir')
    finally:
        os.chdir(initial_dir)

def cvmfs_download():
    url = 'https://github.com/cvmfs/cvmfs/archive/cvmfs-2.3.5.tar.gz'
    #url = 'https://github.com/cvmfs/cvmfs/archive/libcvmfs-stable.tar.gz'
    subprocess.check_call(['wget', url])
    subprocess.check_call(['tar', '-zxf', 'cvmfs-2.3.5.tar.gz'])
    return 'cvmfs-cvmfs-2.3.5'

def cvmfs_build():
    libuuid = libuuid_build()
    dirname = cvmfs_download()
    initial_dir = os.getcwd()
    os.chdir(dirname)
    try:
        if os.path.exists('release_dir'):
            shutil.rmtree('release_dir')
        os.mkdir('release_dir')
        options = ['-Wno-dev',
                   '-DINSTALL_MOUNT_SCRIPTS=OFF',
                   '-DBUILD_SERVER=OFF',
                   '-DBUILD_CVMFS=OFF',
                   '-DBUILD_LIBCVMFS=ON',
                   '-DINSTALL_BASH_COMPLETION=OFF',
                   '-DUUID_LIBRARY:FILE='+os.path.join(libuuid,'lib','libuuid.a'),
                   '-DUUID_INCLUDE_DIR:PATH='+os.path.join(libuuid,'include'),
                   '-DCMAKE_INSTALL_PREFIX='+os.path.join(os.getcwd(),'release_dir'),
                  ]
        subprocess.check_call(['cmake']+options)
        subprocess.check_call(['make','libpacparser'])
        os.chdir('cvmfs')
        subprocess.check_call(['make'])
        subprocess.check_call(['make','install'])
        return os.path.join(initial_dir,dirname,'release_dir')
    finally:
        os.chdir(initial_dir)

def parrot_download(version):
    url = 'http://ccl.cse.nd.edu/software/files/cctools-'+version+'-source.tar.gz'
    subprocess.check_call(['wget', url])
    subprocess.check_call(['tar', '-zxf', 'cctools-'+version+'-source.tar.gz'])
    return 'cctools-'+version+'-source'

def parrot_build(version='6.0.14'):
    cvmfs = cvmfs_build()
    dirname = parrot_download(version)
    initial_dir = os.getcwd()
    os.chdir(dirname)
    try:
        if os.path.exists('release_dir'):
            shutil.rmtree('release_dir')
        os.mkdir('release_dir')
        options = ['--without-system-sand',
                   '--without-system-allpairs',
                   '--without-system-wavefront',
                   '--without-system-makeflow',
                   #'--without-system-ftp-lite',
                   #'--without-system-chirp',
                   '--without-system-umbrella',
                   '--without-system-resource_monitor',
                   '--without-system-doc',
                   '--with-cvmfs-path',cvmfs,
                   '--prefix',os.path.join(os.getcwd(),'release_dir'),
                  ]
        subprocess.check_call(['./configure']+options)
        subprocess.check_call(['make'])
        subprocess.check_call(['make','install'])
        return os.path.join(initial_dir,dirname,'release_dir')
    finally:
        os.chdir(initial_dir)

def condor_download(version):
    version = version.replace('.','_')
    url = 'https://github.com/htcondor/htcondor/archive/V'+version+'.tar.gz'
    subprocess.check_call(['wget', url])
    subprocess.check_call(['tar', '-zxf', 'V'+version+'.tar.gz'])
    return 'htcondor-'+version

def condor_build(version='8.6.1'):
    dirname = condor_download(version)
    initial_dir = os.getcwd()
    os.chdir(dirname)
    try:
        if os.path.exists('release_dir'):
            shutil.rmtree('release_dir')
        os.mkdir('release_dir')
        # skip boost forcefully
        boost_cmakelists = 'externals/bundles/boost/1.49.0/CMakeLists.txt'
        lines = open(boost_cmakelists).read()
        with open(boost_cmakelists,'w') as f:
            for line in lines.split('\n'):
                if line.startswith('if (NOT PROPER)'):
                    f.write('if (FALSE)\n')
                else:
                    f.write(line+'\n')
        options = [
            '-DPROPER=OFF',
            '-DCMAKE_SKIP_RPATH=ON',
            '-DDLOPEN_SECURITY_LIBS=FALSE',
            '-DHAVE_SHARED_PORT=OFF',
            '-DHAVE_BACKFILL=OFF',
            '-DHAVE_BOINC=OFF',
            '-DHAVE_HIBERNATION=OFF',
            '-DHAVE_KBDD=OFF',
            '-DWANT_GLEXEC=OFF',
            '-DWANT_FULL_DEPLOYMENT=OFF',
            '-DWITH_BOINC=OFF',
            '-DWITH_BOSCO=OFF',
            '-DWITH_CAMPUSFACTORY=OFF',
            '-DWITH_BLAHP=OFF',
            '-DWITH_CURL=OFF',
            '-DWITH_COREDUMPER=OFF',
            '-DWITH_CREAM=OFF',
            '-DWITH_GANGLIA=OFF',
            '-DWITH_GLOBUS=ON',
            '-DWITH_GSOAP=OFF',
            '-DWITH_VOMS=OFF',
            '-DWITH_HADOOP=OFF',
            '-DWITH_LIBCGROUP=OFF',
            '-DWITH_POSTGRESQL=OFF',
            '-DWITH_LIBDELTACLOUD=OFF',
            '-DWITH_LIBVIRT=OFF',
            '-DWITH_PYTHON_BINDINGS=OFF',
            '-DWITH_UNICOREGAHP=OFF',
            '-DWITH_QPID=OFF',
            '-DWITH_WSO2=OFF',
        ]
        if version > '8.5.2':
            options.append('-DWITH_KRB5=OFF')
        subprocess.check_call(['cmake','-DCMAKE_INSTALL_PREFIX:PATH='+os.getcwd()+'/release_dir']
                              +options+['.'])
        subprocess.check_call(['make'])
        subprocess.check_call(['make','install'])
        return os.path.join(initial_dir,dirname,'release_dir')
    finally:
        os.chdir(initial_dir)

def main():
    from optparse import OptionParser
    parser = OptionParser()
    parser.add_option('--template-dir',dest='template',default='glidein_template',
                      help='Location of template directory')
    parser.add_option('--htcondor-version',dest='condor',default=None,
                      help='HTCondor version to use')
    parser.add_option('--parrot-version',dest='parrot',default=None,
                      help='Parrot (cctools) version to use')
    parser.add_option('-o','--output',dest='output',default='glidein.tar.gz',
                      help='output tarball name')
    (options, args) = parser.parse_args()
    if not options.template:
        raise Exception('need a template directory')
    options.template = os.path.abspath(options.template)

    curdir = os.getcwd()
    d = tempfile.mkdtemp(dir=os.getcwd())
    tarfile_name = os.path.abspath(os.path.expandvars(os.path.expanduser(options.output)))
    try:
        os.chdir(d)
        parrot_opts = {}
        if options.parrot:
            parrot_opts['version'] = options.parrot
        parrot_path = parrot_build(**parrot_opts)
        condor_opts = {}
        if options.condor:
            condor_opts['version'] = options.condor
        condor_path = condor_build(**condor_opts)
        with tarfile.open(tarfile_name,'w:gz') as tar:
            for f in os.listdir(options.template):
                path = os.path.join(options.template,f)
                if not os.path.isdir(path):
                    os.chmod(path, stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR)
                tar.add(path,arcname=f)
            tar.add('.',arcname='glideinExec',recursive=False)
            for f in os.listdir(condor_path):
                tar.add(os.path.join(condor_path,f),arcname=os.path.join('glideinExec',f))
            tar.add(os.path.join(parrot_path,'bin','parrot_run'),arcname=os.path.join('GLIDEIN_PARROT','parrot_run'))
            tar.add(os.path.join(parrot_path,'lib','libparrot_helper.so'),arcname=os.path.join('GLIDEIN_PARROT','libparrot_helper.so'))
    finally:
        os.chdir(curdir)
        shutil.rmtree(d)

if __name__ == '__main__':
    main()



import logging
import os
import pwd
import shlex

from datetime import datetime, timedelta
from subprocess import check_output, STDOUT
from pyglidein.client_util import Client


class ClientMetrics(object):
    """
    Base class for the client metrics classes.  Provides a framework for collecting client metrics
    and sending them to an endpoint
    """

    def __init__(self, config, secrets):

        if config.get('Cluster', {}).get('user', None) is not None:
            self.user = config['Cluster']['user']
        elif os.environ.get('USER', None) is not None:
            self.user = os.environ['USER']
        else:
            self.user = pwd.getpwuid(os.getuid()).pw_name

        self.address = config['Glidein']['address']

        self.config = config
        self.secrets = secrets
        self.logger = logging.getLogger('client')

    def send(self, metrics_bundle):
        """
        Sending metrics information back
        """
        c = Client(address=self.address)
        try:
            return c.request('monitoring', metrics_bundle.get_bundle())
        except Exception:
            self.logger.warn('error getting state', exc_info=True)


class ClientMetricsCondor(ClientMetrics):
    """
    Collect client metrics from a Condor cluster
    """

    def get_mma_idle_time(self, partition='Cluster'):
        import classad
        import htcondor
        
        schedd = htcondor.Schedd()
        now = datetime.now()
        job_count = 0
        total = timedelta(0)
        min_delta = timedelta.max
        max_delta = timedelta(0)
        metrics = {
            'avg_idle_time': {},
            'min_idle_time': {},
            'max_idle_time': {}
        }

        requirements = 'Owner == "{}" && '.format(self.user)
        requirements += 'JobStatus == 1'
        projection = ['QDate']
        for job in schedd.xquery(requirements=requirements, projection=projection):
            delta = now - datetime.fromtimestamp(job['QDate'])
            if delta < min_delta:
                min_delta = delta
            if delta > max_delta:
                max_delta = delta
            total += delta
            job_count += 1

        # Updating Metrics
        if job_count > 0:
            metrics['avg_idle_time'][partition] = int(total.total_seconds() / job_count)
        else:
            metrics['avg_idle_time'][partition] = 0
        metrics['min_idle_time'][partition] = int(min_delta.total_seconds())
        metrics['max_idle_time'][partition] = int(max_delta.total_seconds())

        self.logger.debug('avg_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['avg_idle_time'][partition]))
        self.logger.debug('min_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['min_idle_time'][partition]))
        self.logger.debug('max_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['max_idle_time'][partition]))

        return metrics


class ClientMetricsSlurm(ClientMetrics):
    """
    Collect client metrics from a Slurm cluster
    """

    def get_mma_idle_time(self, partition='Cluster'):
        DEFAULT_MMA_CMD = 'squeue -u {} -t PENDING -o "%V" -h'.format(self.user)

        now = datetime.now()
        job_count = 0
        total = timedelta(0)
        min_delta = timedelta.max
        max_delta = timedelta(0)
        metrics = {
            'avg_idle_time': {},
            'min_idle_time': {},
            'max_idle_time': {}
        }

        cmd = DEFAULT_MMA_CMD
        if self.config.get(partition, {}).get('mma_cmd', False):
            cmd = self.config[partition]['mma_cmd']
        cmd = os.path.expandvars(cmd)
        cmd = shlex.split(cmd)
        output = check_output(cmd, shell=False, env=os.environ, stderr=STDOUT)
        for line in output.split('\n'):
            if line != '':
                qtime = datetime.strptime(line, '%Y-%m-%dT%H:%M:%S')
                delta = now - qtime
                if delta < min_delta:
                    min_delta = delta
                if delta > max_delta:
                    max_delta = delta
                total += delta
                job_count += 1

        # Updating Metrics
        if job_count > 0:
            metrics['avg_idle_time'][partition] = int(total.total_seconds() / job_count)
            metrics['min_idle_time'][partition] = int(min_delta.total_seconds())
            metrics['max_idle_time'][partition] = int(max_delta.total_seconds())
        else:
            metrics['avg_idle_time'][partition] = 0
            metrics['min_idle_time'][partition] = 0
            metrics['max_idle_time'][partition] = 0

        self.logger.debug('avg_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['avg_idle_time'][partition]))
        self.logger.debug('min_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['min_idle_time'][partition]))
        self.logger.debug('max_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['max_idle_time'][partition]))

        return metrics


class ClientMetricsPBS(ClientMetrics):
    """
    Collect client metrics from a PBS cluster
    """

    def get_mma_idle_time(self, partition='Cluster'):
        DEFAULT_MMA_CMD = 'qstat -u {} -f'.format(self.user)

        now = datetime.now()
        job_count = 0
        total = timedelta(0)
        min_delta = timedelta.max
        max_delta = timedelta(0)
        metrics = {
            'avg_idle_time': {},
            'min_idle_time': {},
            'max_idle_time': {}
        }

        cmd = DEFAULT_MMA_CMD
        if self.config.get(partition, {}).get('mma_cmd', False):
            cmd = self.config[partition]['mma_cmd']
        cmd = os.path.expandvars(cmd)
        cmd = shlex.split(cmd)
        output = check_output(cmd, shell=False, env=os.environ, stderr=STDOUT)
        idle = False
        for line in output.split('\n'):
            if 'job_state = Q' in line:
                idle = True
            elif 'qtime' in line and idle:
                idle = False
                qtime = datetime.strptime(line.split('=')[1].lstrip(), '%a %b %d %H:%M:%S %Y')
                delta = now - qtime
                if delta < min_delta:
                    min_delta = delta
                if delta > max_delta:
                    max_delta = delta
                total += delta
                job_count += 1

        # Updating Metrics
        if job_count > 0:
            metrics['avg_idle_time'][partition] = int(total.total_seconds() / job_count)
        else:
            metrics['avg_idle_time'][partition] = 0
        metrics['min_idle_time'][partition] = int(min_delta.total_seconds())
        metrics['max_idle_time'][partition] = int(max_delta.total_seconds())

        self.logger.debug('avg_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['avg_idle_time'][partition]))
        self.logger.debug('min_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['min_idle_time'][partition]))
        self.logger.debug('max_idle_time for ' +
                          '{}: {}s.'.format(partition, metrics['max_idle_time'][partition]))

        return metrics


class ClientMetricsBundle(object):

    whitelist = {
        'glideins_launched': True,
        'glideins_running': True,
        'glideins_idle': True,
        'avg_idle_time': True,
        'min_idle_time': True,
        'max_idle_time': True
    }

    def __init__(self, uuid, metrics={}, timestamp=None):

        self.metrics = {
            'avg_idle_time': {
                'Cluster': 0
            },
            'min_idle_time': {
                'Cluster': 0
            },
            'max_idle_time': {
                'Cluster': 0
            },
            'glideins_idle': {
                'Cluster': 0
            },
            'glideins_launched': {
                'Cluster': 0
            },
            'glideins_running': {
                'Cluster': 0
            }
        }
        if timestamp is not None:
            self.timestamp = timestamp
        else:
            self.timestamp = int(datetime.utcnow().strftime('%s'))
        self.uuid = uuid

        # Pruning non whitelisted metrics and converting to ints
        self.update_metrics(metrics)

    def get_metrics(self):
        return self.metrics

    def get_timestamp(self):
        return self.timestamp

    def get_uuid(self):
        return self.uuid

    def get_bundle(self):
        bundle = {
            'uuid': self.uuid,
            'timestamp': self.timestamp,
            'metrics': self.metrics
        }
        return bundle

    def get_v1_bundle(self):
        bundle = {
            'timestamp': self.timestamp
        }
        for m in self.metrics:
            bundle[m] = sum(self.metrics[m].values())
        return bundle

    def update_metric(self, metric_name, partition, value):
        if ClientMetricsBundle.whitelist.get(metric_name, False):
            try:
                self.metrics[metric_name][partition] = int(value)
                return True
            except:
                return False

    def update_metrics(self, metrics):
        for k in metrics:
            if ClientMetricsBundle.whitelist.get(k, False):
                # Handling metrics from old clients
                if type(metrics[k]) is not dict:
                    try:
                        self.metrics[k]['Cluster'] = int(metrics[k])
                    except:
                        self.metrics[k]['Cluster'] = 0
                else:
                    for partition in metrics[k]:
                        try:
                            self.metrics[k][partition] = int(metrics[k][partition])
                        except:
                            self.metrics[k][partition] = 0



#!/usr/bin/env python

from optparse import OptionParser
from telnetlib import Telnet


def main():

    usage = "usage: %prog [options] GRIDFTP_SERVER_FQDN GRIDFTP_SERVER_PORT"
    parser = OptionParser(usage)
    parser.add_option('--timeout', type='int', default=5,
                      help="Telnet timeout value in seconds")
    (options, args) = parser.parse_args()

    if len(args) != 2:
        raise Exception('Number Of Args != 2')
    gridftp_server_name = args[0]
    gridftp_server_port = args[1]

    try:
        gridftp_connection = Telnet(gridftp_server_name, gridftp_server_port, options.timeout)
        output = gridftp_connection.read_until('ready.', options.timeout)
        for txt in ['220', 'GridFTP Server', 'ready']:
            if txt not in output:
                raise Exception()
        gridftp_connection.close()

        print 'PYGLIDEIN_RESOURCE_GRIDFTP=True'
        print '- update:true'
    except:
        print 'PYGLIDEIN_RESOURCE_GRIDFTP=False'
        print '- update:true'


if __name__ == '__main__':
    main()



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function

import os
import sys
import time
import subprocess
import logging
import socket
import getpass
from optparse import OptionParser
import stat

from pyglidein.util import json_decode
from pyglidein.client_util import get_state, monitoring
import pyglidein.submit as submit
import pyglidein.client_metrics as client_metrics

from pyglidein.config import Config

logger = logging.getLogger('client')



def get_ssh_state():
    """Getting the state of the remote queue from a text file"""
    try:
        filename = os.path.expanduser('~/glidein_state')
        return json_decode(open(filename).read())
    except Exception:
        logger.warn('error getting ssh state', exc_info=True)

def get_running(cmd):
    """Determine how many jobs are running in the queue"""
    cmd = os.path.expandvars(cmd)
    p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    return int(p.communicate()[0].strip())

def sort_states(state, columns, reverse=True):
    """
    Sort the states according to the list given by prioritize_jobs.

    prioritize_jobs (or columns in this case) is list of according to
    which state should be prioritized for job submission. The position
    in the list indicates the prioritization. columns = ["memory", "disk"]
    with reverse=True means jobs with high memory will be submitted before
    jobs with lower memory requirements, followed by jobs with high disk vs.
    low disk requirement. Jobs with high memory and disk requirements
    will be submitted first then jobs with high memory and medium disk
    requirements, and so on and so forth.

    Args:
        state: List of states
        columns: List of keys in the dict by which dict is sorted
        reverse: Reverse the sorting or not. True = Bigger first,
                 False = smaller first
    """
    key_cache = {}
    col_cache = dict([(c[1:],-1) if c[0] == '-' else (c,1) for c in columns])
    def comp_key(key):
        if key in key_cache:
            return key_cache[key]
        if key in col_cache:
            ret = len(columns)-columns.index(key if col_cache[key] == 1 else '-'+key)
        else:
            ret = 0
        key_cache[key] = ret
        return ret
    def compare(row):
        ret = []
        for k in sorted(row, key=comp_key, reverse=True):
            v = row[k]
            if k in col_cache:
                v *= col_cache[k]
            ret.append(v)
        return ret
    return sorted(state, key=compare, reverse=reverse)


def main():
    parser = OptionParser()
    parser.add_option('--config', type='string', default='cluster.config',
                      help="config file for cluster")
    parser.add_option('--secrets', type='string', default='.pyglidein_secrets',
                      help="secrets file for cluster")
    parser.add_option('--uuid', type='string',
                      default=getpass.getuser()+'@'+socket.gethostname(),
                      help="Unique id for this client")
    (options, args) = parser.parse_args()

    config_dict = Config(options.config)
    config_glidein = config_dict['Glidein']
    config_cluster = config_dict['Cluster']
    if 'StartdLogging' in config_dict:
        config_startd_logging = config_dict['StartdLogging']
    else:
        config_startd_logging = {}

    if ('Mode' in config_dict and 'debug' in config_dict['Mode'] and
        config_dict['Mode']['debug']):
        logging.basicConfig(level=logging.DEBUG,format='%(asctime)s %(levelname)s %(message)s')
    else:
        logging.basicConfig(level=logging.INFO,format='%(asctime)s %(levelname)s %(message)s')

    # Loading secrets.  Fail if permissions wrong.
    if os.path.isfile(options.secrets):
        if os.stat(options.secrets).st_mode & (stat.S_IXGRP | stat.S_IRWXO):
            logger.error('Set Permissions on {} to 600'.format(options.secrets))
            sys.exit(1)
        secrets_dict = Config(options.secrets)
        if 'StartdLogging' in secrets_dict:
            secrets_startd_logging = secrets_dict['StartdLogging']
        else:
            secrets_startd_logging = {}
    else:
        logger.error('Error Accessing Secrets File: {}.  '.format(options.secrets) +
                     'Did you set the --secrets flag?')
        sys.exit(1)

    # Importing the correct class to handle the submit
    sched_type = config_cluster["scheduler"].lower()
    if sched_type == "htcondor":
        scheduler = submit.SubmitCondor(config_dict, secrets_dict)
        metrics = client_metrics.ClientMetricsCondor(config_dict, secrets_dict)
    elif sched_type == "pbs":
        scheduler = submit.SubmitPBS(config_dict, secrets_dict)
        metrics = client_metrics.ClientMetricsPBS(config_dict, secrets_dict)
    elif sched_type == "slurm":
        scheduler = submit.SubmitSLURM(config_dict, secrets_dict)
        metrics = client_metrics.ClientMetricsSlurm(config_dict, secrets_dict)
    elif sched_type == "uge":
        scheduler = submit.SubmitUGE(config_dict, secrets_dict)
        metrics = client_metrics.ClientMetricsPBS(config_dict, secrets_dict)
    elif sched_type == "lsf":
        scheduler = submit.SubmitLSF(config_dict, secrets_dict)
        metrics = client_metrics.ClientMetricsPBS(config_dict, secrets_dict)
    else:
        raise Exception('scheduler not supported')

    # if "glidein_cmd" not in config_dict["Glidein"]:
    #     raise Exception('no glidein_cmd')

    # Failing if startd logging is enabled and python version < 2.7
    if ('send_startd_logs' in config_startd_logging and
        config_startd_logging['send_startd_logs'] is True and
        sys.version_info < (2, 7)):
        logger.error('Python version must be > 2.7 to enable startd logging.')
        sys.exit(1)
    # Checking on startd logging configuration if enabled
    if ('send_startd_logs' in config_startd_logging and
        config_startd_logging['send_startd_logs'] is True):
        for config_val in ['url', 'bucket']:
            if config_val not in config_startd_logging:
                logger.error('Missing %s configuration value in StartdLogging Section' % config_val)
                sys.exit(1)
        for secret_val in ['access_key', 'secret_key']:
            if secret_val not in secrets_startd_logging:
                logger.error('Missing %s secret value in StartdLogging Section' % secret_val)
                sys.exit(1)

    while True:
        if 'ssh_state' in config_glidein and config_glidein['ssh_state']:
            state = get_ssh_state()
        else:
            state = get_state(config_glidein['address'])
        if 'uuid' in config_glidein:
            options.uuid = config_glidein['uuid']
        info = {'uuid': options.uuid,
                'glideins_idle': dict(),
                'glideins_running': dict(),
                'glideins_launched': dict(),
               }
        metrics_bundle = client_metrics.ClientMetricsBundle(options.uuid)
        if state:
            for partition in config_dict['Cluster'].get('partitions', ['Cluster']):
                config_cluster = config_dict[partition]
                if "running_cmd" not in config_cluster:
                    raise Exception('Section [%s] has no running_cmd' % partition)
                idle = 0
                try:
                    info['glideins_running'][partition] = get_running(config_cluster["running_cmd"])
                    metrics_bundle.update_metric('glideins_running', partition,
                                                 info['glideins_running'][partition])
                    if "idle_cmd" in config_cluster:
                        idle = get_running(config_cluster["idle_cmd"])
                        info['glideins_idle'][partition] = idle
                        metrics_bundle.update_metric('glideins_idle', partition,
                                                     info['glideins_idle'][partition])
                except Exception:
                    logger.warn('error getting running job count', exc_info=True)
                    continue
                info['glideins_launched'][partition] = 0
                limit = min(config_cluster["limit_per_submit"],
                            config_cluster["max_total_jobs"] - info['glideins_running'][partition],
                            max(config_cluster.get("max_idle_jobs", 1000) - idle, 0))
                # Prioitize job submission. By default, prioritize submission of gpu and high memory jobs.
                state = sort_states(state, config_cluster["prioritize_jobs"])
                for s in state:
                    if sched_type == "pbs":
                        s["memory"] = s["memory"]*1024/1000
                    if limit <= 0:
                        logger.info('reached limit')
                        break
                    # Skipping CPU jobs for gpu only clusters
                    if ('gpu_only' in config_cluster and config_cluster['gpu_only']
                        and s["gpus"] == 0):
                        continue
                    # skipping GPU jobs for cpu only clusters
                    if ('cpu_only' in config_cluster and config_cluster['cpu_only']
                        and s["gpus"] != 0):
                        continue
                    # skipping jobs over cluster resource limits
                    if config_cluster['whole_node']:
                        prefix = 'whole_node_%s'
                    else:
                        prefix = 'max_%s_per_job'
                    for resource in ('cpus','gpus','memory','disk'):
                        cfg_name = prefix%resource
                        if (cfg_name in config_cluster
                            and s[resource] > config_cluster[cfg_name]):
                            break
                        cfg_name = 'min_%s_per_job'%resource
                        if (cfg_name in config_cluster
                            and s[resource] < config_cluster[cfg_name]):
                            break
                    else:
                        if "count" in s and s["count"] > limit:
                            s["count"] = limit
                        scheduler.submit(s, partition)
                        num = 1 if "count" not in s else s["count"]
                        limit -= num
                        info['glideins_launched'][partition] += num
                metrics_bundle.update_metric('glideins_launched', partition,
                                             info['glideins_launched'][partition])
                logger.info('launched %d glideins on %s', info['glideins_launched'][partition], partition)
        else:
            logger.info('no state, nothing to do')

        metrics_bundle.update_metrics(metrics.get_mma_idle_time())
        metrics.send(metrics_bundle)

        if 'delay' not in config_glidein or int(config_glidein['delay']) < 1:
            break
        time.sleep(config_glidein['delay'])
    for partition in config_dict['Cluster'].get('partitions', ['Cluster']):
        config_cluster = config_dict[partition]
        if "cleanup" in config_cluster and config_cluster["cleanup"]:
            scheduler.cleanup(config_cluster["running_cmd"], config_cluster["dir_cleanup"])


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages

setup(
    name='pyglidein',
    use_scm_version=True,
    setup_requires=['setuptools_scm'],
    description='Some python scripts to launch HTCondor glideins',
    url='https://github.com/WIPACrepo/pyglidein',
    author='WIPAC',
    author_email='contact-us@icecube.wisc.edu',
    classifiers=[
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='htcondor dHTC glidein',
    packages=find_packages(),
    install_requires=['minio;python_version>"2.6"', 'tornado'],
    package_data={
        'pyglidein': ['etc/client_defaults.cfg',
                      'glidein_start.sh', 'log_shipper.sh', 'os_arch.sh',
                      'startd_cron_scripts/clsim_gpu_test.py',
                      'startd_cron_scripts/cvmfs_test.py',
                      'startd_cron_scripts/gridftp_test.py',
                      'startd_cron_scripts/post_cvmfs.sh',
                      'startd_cron_scripts/pre_cvmfs.sh']
    },
    entry_points={
        'console_scripts': [
            'pyglidein_client=pyglidein.client:main',
            'pyglidein_server=pyglidein.server:main'
        ]
    },
    python_requires='>=2.6'
)



from ConfigParser import SafeConfigParser

import os
import ast


class Config(dict):
    def __init__(self, path, default=os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                                  'etc/client_defaults.cfg')):
        self.path = path

        # read defaults
        tmp = SafeConfigParser()
        tmp.optionxform = str
        tmp.read(default)
        self._config_options_dict(tmp)
        
        # read file
        tmp = SafeConfigParser()
        tmp.optionxform = str
        tmp.read(path)
        self._config_options_dict(tmp)
        self._populate_partitions()


    def _config_options_dict(self, config):
        """
        Parsing config file
        Args:
            config: Python config parser object
        """
        for section in config.sections():
            if section not in self:
                self[section] = {}
            for option in config.options(section):
                val = config.get(section, option)
                try:
                    val = ast.literal_eval(val)
                except Exception as e:
                    pass
                self[section][option] = val

    def _populate_partitions(self):
        cluster_config = self.get('Cluster', dict())
        if 'partitions' in cluster_config:
            cluster_config['partitions'] = [k.strip() for k in cluster_config['partitions'].split(',')]
            for k in cluster_config['partitions']:
                if not k in self:
                    continue
                config = dict(cluster_config)
                config.update(self[k])
                self[k] = config



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function

import os
import subprocess
import logging
import shutil
import glob
import uuid

from pyglidein.client_util import get_presigned_put_url, get_presigned_get_url


class Submit(object):
    """
    Base class for the submit classes
    Mostly to provide future expansion for common functions
    """
    def __init__(self, config, secrets):
        """
        Initialize

        Args:
            config: cluster config dict for cluster
            secrets: cluster secrets dict for cluster
        """
        self.config = config
        self.secrets = secrets
        self.startd_cron_scripts = ['clsim_gpu_test.py',
                                    'cvmfs_test.py',
                                    'gridftp_test.py',
                                    'post_cvmfs.sh',
                                    'pre_cvmfs.sh']

    def submit(self):
        raise NotImplementedError()

    def write_line(self, f, line):
        """
        Wrapper function so we dont have to write \n a million times

        Args:
            f: File handle
            line: Line to be written to file
        """
        f.write(line+"\n")

    def get_glidein_script(self):
        """
        Get the glidein startup script.

        Returns:
            String that is the location of the script
        """
        # If the user hasn't set ['Glidein']['glidein_script'] assume they want to use the
        # glidein_scripts provided by the python package.
        package_dir = os.path.dirname(os.path.abspath(__file__))
        glidein_script = os.path.join(package_dir, 'glidein_start.sh')
        
        if 'glidein_script' in self.config['Glidein']:
            glidein_script = self.config['Glidein']['glidein_script']
        return glidein_script

    def get_resource_limit_scale(self, key, sec="SubmitFile"):
        """
        Return scaling factor for job limit resources
        (e.g. memory, disk space).

        Args:
            key: key to evaluate in config file
            sec: section to evaluate in config file
                 (default: 'SubmitFile')
        Returns:
            A float that is the scaling factor for a resource
        """
        try:
            # look for entry and check type
            scale = self.config[sec][key]
            if not (isinstance(scale, int) or
                    isinstance(scale, float)):
                raise TypeError()
        except:
            # return 1 if no entry or invalid type found
            scale = 1

        return scale

    def cleanup(self, cmd, direc):
        pass


class SubmitPBS(Submit):
    """Submit a PBS / Torque job"""

    option_tag = "#PBS"

    def write_option(self, f, line):
        self.write_line(f, self.option_tag+" "+line)

    def write_general_header(self, f, cluster_config, mem=3000, walltime_hours=14, disk=1,
                             num_nodes=1, num_cpus=1, num_gpus=0,
                             num_jobs=0):
        """
        Writing the header for a PBS submission script.
        Most of the pieces needed to tell PBS what resources
        are being requested.

        Args:
            f: python file object
            mem: requested memory
            walltime_hours: requested wall time
            num_nodes: requested number of nodes
            num_cpus: requested number of cpus
            num_gpus: requested number of gpus
        """
        self.write_line(f, "#!/bin/bash")
        # Add the necessary gpu request tag if we need gpus.

        resource_line = "-l nodes=%d:ppn=%d" % (num_nodes, num_cpus)
        node_property = cluster_config.get("node_property", False)
        if node_property:
            resource_line +=':%s' % (node_property)
        if num_gpus > 0 and cluster_config.get("set_gpu_req", True):
            resource_line += ':gpus=%d' % (num_gpus)
        self.write_option(f, resource_line)

        if cluster_config.get("pmem_only", False):
            self.write_option(f, "-l pmem=%dmb" % mem)
        elif cluster_config.get("pvmem", False):
            self.write_option(f, "-l pmem=%dmb,vmem=%dmb" % (mem,mem*num_cpus))
        elif cluster_config.get("vmem_only", False):
            self.write_option(f, "-l vmem=%dmb" % mem)
        elif cluster_config.get("mem_only", False):
            self.write_option(f, "-l mem=%dmb" % mem)
        else:
            self.write_option(f, "-l pmem=%dmb,mem=%dmb" % (mem,mem*num_cpus))
        self.write_option(f, "-l walltime=%d:00:00" % walltime_hours)
        if ('Mode' in self.config and 'debug' in self.config['Mode']
           and self.config["Mode"]["debug"]):
            outdir = os.path.join(os.getcwd(),'out')
            if not os.path.isdir(outdir):
                os.mkdir(outdir)
            self.write_option(f, "-o %s/${PBS_JOBID}.out"%outdir)
            self.write_option(f, "-e %s/${PBS_JOBID}.err"%outdir)
        else:
            self.write_option(f, "-o /dev/null")
            self.write_option(f, "-e /dev/null")
        if num_jobs > 0:
            self.write_option(f, "-t 0-%d" % num_jobs)
        env_vars = '-v '
        if not self.config.get("StartdChecks", {}).get("enable_startd_checks", True):
            env_vars += 'DISABLE_STARTD_CHECKS=1'
        if env_vars != '-v ':
            self.write_option(f, env_vars)

    def write_glidein_variables(self, f, mem=1000, walltime_hours=12,
                                num_cpus=1, num_gpus=0, disk=1):
        """
        Tell the glidein what resources it has.

        Args:
            f: python file object
            mem: memory provided for glidein
            walltime_hours: lifetime of glidein in hours
            cpus: number of cpus provided
            gpus: number of cpus provided
            disk: disk provided for glidein
        """
        self.write_line(f, "MEMORY=%d" % mem)
        # we do not want same walltime as this may lead to hitting the walltime, thus we spot the glidein 5 min earlier (300 sec)
        self.write_line(f, "WALLTIME=%d" % (walltime_hours*3600 - 300))
        self.write_line(f, "CPUS=%d" % num_cpus)
        self.write_line(f, "DISK=%d" % (disk*1024))
        if num_gpus:
            self.write_line(f, 'if [ "$CUDA_VISIBLE_DEVICES" -eq "$CUDA_VISIBLE_DEVICES" ] 2>/dev/null ; then')
            self.write_line(f, '  GPUS="CUDA${CUDA_VISIBLE_DEVICES}"')
            self.write_line(f, 'elif [ "x$CUDA_VISIBLE_DEVICES" = "x" ] ; then')
            self.write_line(f, '  GPUS=%d'%num_gpus)
            self.write_line(f, 'else')
            self.write_line(f, '  GPUS=$CUDA_VISIBLE_DEVICES')
            self.write_line(f, 'fi')
        else:
            self.write_line(f, 'GPUS=0')

        if 'site' in self.config['Glidein']:
            self.write_line(f, 'SITE="%s"' % self.config['Glidein']['site'])
        if 'resourcename' in self.config['Glidein']:
            self.write_line(f, 'ResourceName="%s"' % self.config['Glidein']['resourcename'])
        if 'cluster' in self.config['Glidein']:
            self.write_line(f, 'CLUSTER="%s"' % self.config['Glidein']['cluster'])

    def write_glidein_part(self, f, local_dir=None, glidein_tarball=None, presigned_put_url=None,
                           presigned_get_url=None):
        """
        Writing the pieces needed to execute the glidein

        Args:
            f: python file object
            local_dir: what is the local directory
            glidein_tarball: file name of tarball
        """
        self.write_line(f, 'CLEANUP=0')
        self.write_line(f, 'LOCAL_DIR=%s' % local_dir)
        self.write_line(f, 'if [ ! -d $LOCAL_DIR ]; then')
        self.write_line(f, '    mkdir -p $LOCAL_DIR')
        self.write_line(f, '    CLEANUP=1')
        self.write_line(f, 'fi')
        self.write_line(f, 'cd $LOCAL_DIR')
        if glidein_tarball:
            self.write_line(f, 'ln -fs %s %s' % (glidein_tarball, os.path.basename(glidein_tarball)))
        glidein_script = self.get_glidein_script()
        if not os.path.isfile(glidein_script):
            raise Exception("glidein_script %s does not exist!" % glidein_script)
        self.write_line(f, 'ln -fs %s %s' % (glidein_script, os.path.basename(glidein_script)))
        osarch_script = os.path.join(os.path.dirname(glidein_script), 'os_arch.sh')
        if not os.path.isfile(osarch_script):
            raise Exception("%s does not exist!" % osarch_script)
        self.write_line(f, 'ln -fs %s %s' % (osarch_script, 'os_arch.sh'))
        log_shipper_script = os.path.join(os.path.dirname(glidein_script), 'log_shipper.sh')
        if not os.path.isfile(log_shipper_script):
            raise Exception("%s does not exist!" % log_shipper_script)
        self.write_line(f, 'ln -fs %s %s' % (log_shipper_script, 'log_shipper.sh'))
        # Adding StartD Cron Scripts
        if self.config.get("StartdChecks", {}).get("enable_startd_checks", True):
            startd_cron_scripts_dir = os.path.join(os.path.dirname(glidein_script),
                                                   'startd_cron_scripts')
            if not os.path.isdir(startd_cron_scripts_dir):
                raise Exception("StartD cron scripts directory not found: "
                                "{}".format(startd_cron_scripts_dir))
            for script in self.startd_cron_scripts:
                script_path = os.path.join(startd_cron_scripts_dir, script)
                if not os.path.isfile(script_path):
                    raise Exception("Stard cron script not found: {}".format(script))
                self.write_line(f, 'ln -fs %s %s' % (script_path, script))
        f.write('exec env -i CPUS=$CPUS GPUS=$GPUS MEMORY=$MEMORY DISK=$DISK WALLTIME=$WALLTIME '
                'DISABLE_STARTD_CHECKS=$DISABLE_STARTD_CHECKS ')
        if 'site' in self.config['Glidein']:
            f.write('SITE=$SITE ')
        f.write('ResourceName=ResourceName ')
        if 'cluster' in self.config['Glidein']:
            f.write('CLUSTER=$CLUSTER ')
        if self.config['SubmitFile'].get('cvmfs_job_wrapper', False):
            f.write('CVMFS_JOB_WRAPPER=1 ')
        if presigned_put_url is not None and presigned_get_url is not None:
            f.write('PRESIGNED_PUT_URL="{}" PRESIGNED_GET_URL="{}" '.format(presigned_put_url,
                                                                            presigned_get_url))
        if "CustomEnv" in self.config:
            for k, v in self.config["CustomEnv"].items():
                f.write(k + '=' + v + ' ')
        executable = os.path.basename(glidein_script)
        self.write_line(f, './%s' % executable)
        self.write_line(f, 'if [ $CLEANUP -eq 1 ]; then')
        self.write_line(f, '    rm -rf $LOCAL_DIR')
        self.write_line(f, 'fi')

    def get_cores_for_memory(self, cluster_config, num_cpus_advertised, num_gpus_advertised, mem_advertised):
        """
        Scale number of cores to satisfy memory request, assuming fixed amount
        of memory per core.

        Args:
            num_cpus_advertised: the number of cores explicitly requested
            num_gpus_advertised: the number of GPUs explicitly requested
            mem_advertised: the total amount of memory requested
        Returns:
            num_cpus: number of cores to request
            mem_requested: amount of memory to request per core
            mem_advertised: amount of memory the Condor slot should advertise
        """
        num_cpus = num_cpus_advertised
        mem_requested = mem_advertised
        mem_per_core = cluster_config.get('mem_per_core', 2000)
        if num_gpus_advertised:
            if mem_requested > mem_per_core:
                # just ask for the max mem, and hope that's good enough
                mem_requested = mem_per_core
                mem_advertised = mem_requested
        else:
            # It is easier to request more cpus rather than more memory
            while mem_requested > mem_per_core:
                num_cpus += 1
                mem_requested = mem_advertised/num_cpus

        return num_cpus, mem_requested, mem_advertised

    def write_submit_file(self, filename, state, group_jobs, cluster_config,
                          presigned_put_url=None, presigned_get_url=None):
        """
        Writing the submit file

        Args:
            filename: name of PBS script to create
            state: what resource requirements a given glidein has
            group_jobs: if True, group jobs into arrays
            cluster_config: the Cluster config dict (or that of an alternate partition)
        """
        with open(filename, 'w') as f:
            if cluster_config['whole_node']:
                num_cpus = int(cluster_config['whole_node_cpus'])
                mem_advertised = int(cluster_config['whole_node_memory'])
                disk = int(cluster_config['whole_node_disk'])
                if 'whole_node_gpus' in cluster_config:
                    num_gpus = int(cluster_config['whole_node_gpus'])
                else:
                    num_gpus = 0
                if 'mem_per_core' not in cluster_config:
                    # by default assume we gave the correct amount in whole_node_memory
                    cluster_config['mem_per_core'] = 100000
                _, mem_requested, mem_advertised = self.get_cores_for_memory(cluster_config, 1, num_gpus, mem_advertised)
            else:
                num_cpus = state["cpus"]
                mem_safety_margin = 1.05*self.get_resource_limit_scale("mem_safety_scale")
                mem_advertised = int(state["memory"]*mem_safety_margin)
                num_gpus = state["gpus"]
                disk = state["disk"]*1.1

                num_cpus, mem_requested, mem_advertised = self.get_cores_for_memory(cluster_config, num_cpus, num_gpus, mem_advertised)

            walltime = int(cluster_config["walltime_hrs"])

            self.write_general_header(f, cluster_config, mem=mem_requested, num_cpus=num_cpus,
                                      num_gpus=num_gpus, walltime_hours=walltime,
                                      disk=disk,
                                      num_jobs = state["count"] if group_jobs else 0)

            if "custom_header" in self.config["SubmitFile"]:
                self.write_line(f, self.config["SubmitFile"]["custom_header"])
            if "custom_middle" in self.config["SubmitFile"]:
                self.write_line(f, self.config["SubmitFile"]["custom_middle"])

            self.write_glidein_variables(f, mem=mem_advertised,
                                         num_cpus=num_cpus, num_gpus=num_gpus,
                                         walltime_hours=walltime, disk=disk)

            kwargs = {
                'local_dir': self.config["SubmitFile"]["local_dir"],
                'presigned_put_url': presigned_put_url,
                'presigned_get_url': presigned_get_url
            }
            if "tarball" in self.config["Glidein"]:
                if "loc" in self.config["Glidein"]:
                    glidein_tarball = os.path.join(self.config["Glidein"]["loc"],
                                                   self.config["Glidein"]["tarball"])
                else:
                    glidein_tarball = self.config["Glidein"]["tarball"]

                if os.path.isfile(glidein_tarball):
                    kwargs['glidein_tarball'] = glidein_tarball
                else:
                    raise Exception("The tarball you provided does not exist")
            self.write_glidein_part(f, **kwargs)

            if "custom_end" in self.config["SubmitFile"]:
                self.write_line(f, self.config["SubmitFile"]["custom_end"])

    def submit(self, state, partition="Cluster"):
        """
        Writing submit file and submitting a job for PBS-like batch managers

        Args:
            state: what resource requirements a given glidein has
        """
        submit_filename = 'submit.pbs'
        if 'filename' in self.config["SubmitFile"]:
            submit_filename = self.config["SubmitFile"]["filename"]

        cluster_config = self.config[partition]
        group_jobs = ("group_jobs" in cluster_config and
                      cluster_config["group_jobs"] and
                      "count" in state)

        self.write_submit_file(submit_filename, state, group_jobs, cluster_config)
        num_submits = 1 if group_jobs else state["count"] if "count" in state else 1
        for i in xrange(num_submits):
            if self.config['StartdLogging']['send_startd_logs'] is True:
                startd_logfile_name = '{}_{}.tar.gz'.format(self.config['Glidein']['site'], uuid.uuid4())
                presigned_put_url = get_presigned_put_url(startd_logfile_name, self.config, self.secrets)
                presigned_get_url = get_presigned_get_url(startd_logfile_name, self.config, self.secrets)
                self.write_submit_file(submit_filename,
                                       state,
                                       group_jobs,
                                       cluster_config,
                                       presigned_put_url,
                                       presigned_get_url)
            cmd = self.config[partition]["submit_command"] + " " + submit_filename
            if not ('Mode' in self.config and 'dryrun' in self.config['Mode'] and
                    self.config['Mode']['dryrun']):
                subprocess.check_call(cmd, shell=True)


    def cleanup(self, cmd, direc):
        """
        Cleans up temporary directories that were created on a network file system that were not
        deleted by the job itself. Checks whether the job ID used to identify a temporary directory
        is still in the queue. If it is not, the directory gets deleted.

        Args:
            cmd: Command needed to query about which jobs are running for the user
            direc: Which directory to look for the temporory directories
        """
        cmd = cmd[:-6]
        p = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
        d = p.communicate()[0]
        job_ids = set([job.split(" ")[0] for job in d.splitlines() if "[" not in job.split(" ")[0] ])
        dir_ids = set([dir.split("/")[-1].split(".")[0] for dir in glob.glob(os.path.join(os.path.expandvars(direc), "*"))])
        for ids in (dir_ids - job_ids):
            logging.info("Deleting %s", ids)
            shutil.rmtree(glob.glob(os.path.join(os.path.expandvars(direc), ids + "*"))[0])

class SubmitSLURM(SubmitPBS):
    """SLURM is similar to PBS, but with different headers"""

    option_tag = "#SBATCH"

    def write_general_header(self, f, cluster_config, mem=3000, walltime_hours=14, disk=1,
                             num_nodes=1, num_cpus=1, num_gpus=0,
                             num_jobs=0):
        """
        Writing the header for a SLURM submission script.
        Most of the pieces needed to tell SLURM what resources
        are being requested.

        Args:
            f: python file object
            mem: requested memory
            walltime_hours: requested wall time
            num_nodes: requested number of nodes
            num_cpus: requested number of cpus
            num_gpus: requested number of gpus
            num_jobs: requested number of jobs
        """
        if num_jobs > 1:
            raise Exception('more than one job not supported')
        self.write_line(f, "#!/bin/bash")
        self.write_option(f, '--job-name="glidein"')
        self.write_option(f, '--nodes=%d'%num_nodes)
        self.write_option(f, '--ntasks-per-node=%d'%num_cpus)
        self.write_option(f, '--mem=%d'%(mem))
        if num_gpus:
            gpu_submit = '--gres=gpu:%d'
            if 'gpu_submit' in self.config['SubmitFile']:
                gpu_submit = self.config['SubmitFile']['gpu_submit']
            self.write_option(f, gpu_submit%num_gpus)
        if "partition" in cluster_config:
            self.write_option(f, "--partition=%s" % cluster_config["partition"])
        self.write_option(f, "--time=%d:00:00" % walltime_hours)
        if self.config["Mode"]["debug"]:
            log_dir = os.path.join(os.getcwd(), 'out')
            if not os.path.isdir(log_dir):
                os.mkdir(log_dir)
            self.write_option(f, "--output="+os.path.join(log_dir, "%j.out"))
            self.write_option(f, "--error="+os.path.join(log_dir, "%j.err"))
        else:
            self.write_option(f, "--output=/dev/null")
            self.write_option(f, "--error=/dev/null")
        self.write_option(f, "--export=ALL")

class SubmitUGE(SubmitPBS):
    """UGE is similar to PBS, but with different headers"""

    option_tag = "#$"

    def get_cores_for_memory(self, cluster_config, num_cpus_advertised, num_gpus_advertised, mem_advertised):
        """
        Scale number of cores to satisfy memory request.

        UGE can assign variable memory per core, so just pass the request straight through.
        """
        return num_cpus_advertised, mem_advertised, mem_advertised

    def write_general_header(self, f, cluster_config, mem=3000, walltime_hours=14, disk=1,
                             num_nodes=1, num_cpus=1, num_gpus=0,
                             num_jobs=0):
        """
        Writing the header for a SLURM submission script.
        Most of the pieces needed to tell SLURM what resources
        are being requested.

        Args:
            f: python file object
            mem: requested memory
            walltime_hours: requested wall time
            num_nodes: requested number of nodes
            num_cpus: requested number of cpus
            num_gpus: requested number of gpus
        """
        self.write_line(f, "#!/bin/bash")
        self.write_option(f, '-S /bin/bash')
        self.write_option(f, '-l h_rss=%dM'%(mem//num_cpus))
        self.write_option(f, '-l tmpdir_size=%dM'%(max((disk//num_cpus, 1000))))
        if num_gpus:
            self.write_option(f, "-l gpu=%d"%num_gpus)
        if num_cpus > 1:
            self.write_option(f, "-pe multicore %d"%num_cpus)
        self.write_option(f, "-l h_rt=%d:00:00" % walltime_hours)
        if self.config["Mode"]["debug"]:
            self.write_option(f, "-o %s/out/$JOB_ID.out"%os.getcwd())
            self.write_option(f, "-e %s/out/$JOB_ID.err"%os.getcwd())
        else:
            self.write_option(f, "-o /dev/null")
            self.write_option(f, "-e /dev/null")
        if num_jobs > 0:
            self.write_option(f, "-t 1-%d" % num_jobs)

class SubmitLSF(SubmitPBS):
    """LSF is similar to PBS, but with different headers"""

    option_tag = "#BSUB"

    def write_general_header(self, f, cluster_config, mem=3000, walltime_hours=14, disk=1,
                             num_nodes=1, num_cpus=1, num_gpus=0,
                             num_jobs=0):
        """
        Writing the header for an LSF submission script.
        Most of the pieces needed to tell LSF what resources
        are being requested.

        Args:
            f: python file object
            mem: requested memory
            walltime_hours: requested wall time
            num_nodes: requested number of nodes
            num_cpus: requested number of cpus
            num_gpus: requested number of gpus
            num_jobs: number of jobs in a job array
        """
        self.write_line(f, "#!/bin/bash")
        #if num_gpus > 0:
        #    self.write_option(f, "-R 'rusage[cuda=%d]'" % num_gpus)
        walltime_line = "-W %d:00" % walltime_hours

        # check for additional parameters in config
        if 'SubmitFile' in self.config:
            submit_conf = self.config['SubmitFile']
            if 'ref_host' in submit_conf:
                # add reference host for walltime if given
                walltime_line+="/%s" % submit_conf['ref_host']

        self.write_option(f, walltime_line)
        # default memory units are kB for LSF
        mem_scale = 1000
        # scale memory to non-default units if parameter exists
        mem_scale*=self.get_resource_limit_scale("mem_scale")
        self.write_option(f, "-M %d" % (mem*mem_scale))
        self.write_option(f, "-n %d" % num_cpus)
        """
        # ignore for now
        # need to make sure to reserve the correct number of nodes
        cpus_tot = num_cpus
        if num_nodes > 1:
            cpus_tot = num_nodes*cpus_per_node
        self.write_option(f, "-n %d -R 'span[ptile=%d]'" %\
                             (cpus_tot, cpus_per_node))
        """
        if num_jobs > 0:
            # job name will be "[index]"
            self.write_option(f, "-J [1-%d]" % num_jobs)

        if ('Mode' in self.config and 'debug' in self.config['Mode']
            and self.config['Mode']['debug']):
            outdir = os.path.join(os.getcwd(), 'out')
            if not os.path.isdir(outdir):
                os.mkdir(outdir)
            # %I is job index (all jobs in an array have same id %J)
            self.write_option(f, "-o %s/%%J_%%I.out" % outdir)
            self.write_option(f, "-e %s/%%J_%%I.err" % outdir)
        else:
            self.write_option(f, "-o /dev/null")
            self.write_option(f, "-e /dev/null")

class SubmitCondor(Submit):
    """Submit an HTCondor job"""

    def make_env_wrapper(self, env_wrapper, cluster_config):
        """
        Creating wrapper execute script for
        HTCondor submit file

        Args:
            env_wrapper: name of wrapper script
        """
        with open(env_wrapper, 'w') as f:
            self.write_line(f, '#!/bin/sh')
            self.write_line(f, 'CPUS=$(grep -e "^Cpus" $_CONDOR_MACHINE_AD|awk -F "= " "{print \\$2}")')
            self.write_line(f, 'MEMORY=$(grep -e "^Memory" $_CONDOR_MACHINE_AD|awk -F "= " "{print \\$2}")')
            self.write_line(f, 'DISK=$(grep -e "^Disk =" $_CONDOR_MACHINE_AD|awk -F "= " "{print \\$2}")')
            self.write_line(f, 'GPUS=$(grep -e "^AssignedGPUs" $_CONDOR_MACHINE_AD|awk -F "= " "{print \\$2}"|sed "s/\\"//g")')
            self.write_line(f, 'if ( [ -z $GPUS ] && [ ! -z $CUDA_VISIBLE_DEVICES ] ); then')
            self.write_line(f, '  GPUS=$CUDA_VISIBLE_DEVICES')
            self.write_line(f, 'fi')
            self.write_line(f, 'GPUS_NO_DIGITS=$(echo $GPUS | sed \'s/[0-9]*//g\')')
            self.write_line(f, 'if [ "${GPUS_NO_DIGITS}" = "${GPUS}" ]; then')
            self.write_line(f, '    GPUS=""')
            self.write_line(f, 'elif [ -z $GPUS_NO_DIGITS ]; then')
            self.write_line(f, '    GPUS="CUDA${GPUS}"')
            self.write_line(f, 'fi')
            self.write_line(f, 'if ( [ -z $GPUS ] || [ "$GPUS" = "10000" ] || [ "$GPUS" = "CUDA10000" ] ); then')
            self.write_line(f, '  GPUS=0')
            self.write_line(f, 'fi')
            if 'site' in self.config['Glidein']:
                self.write_line(f, 'SITE="%s"' % self.config['Glidein']['site'])
            if 'resourcename' in self.config['Glidein']:
                self.write_line(f, 'ResourceName="%s"' % self.config['Glidein']['resourcename'])
            else:
                self.write_line(f, 'ResourceName=$(grep -e "^GLIDEIN_ResourceName" $_CONDOR_MACHINE_AD|awk -F "= " "{print \\$2}"|sed "s/\\"//g")')
            if 'cluster' in self.config['Glidein']:
                self.write_line(f, 'CLUSTER="%s"' % self.config['Glidein']['cluster'])
            f.write('exec env -i CPUS=$CPUS GPUS=$GPUS MEMORY=$MEMORY DISK=$DISK '
                    'PRESIGNED_PUT_URL=$PRESIGNED_PUT_URL PRESIGNED_GET_URL=$PRESIGNED_GET_URL ')
            if 'site' in self.config['Glidein']:
                f.write('SITE=$SITE ')
            f.write('ResourceName=$ResourceName ')
            if 'cluster' in self.config['Glidein']:
                f.write('CLUSTER=$CLUSTER ')
            # reduce walltime for internal job by 300 sec (5min) to make sure it finished before the outer job hits the walltime
            walltime = int(cluster_config["walltime_hrs"])*3600-300
            f.write('WALLTIME=%d '%walltime)
            if self.config['SubmitFile'].get('cvmfs_job_wrapper', False):
                f.write('CVMFS_JOB_WRAPPER=1 ')
            if "CustomEnv" in self.config:
                for k, v in self.config["CustomEnv"].items():
                    f.write(k + '=' + v + ' ')
            if 'executable' in self.config['SubmitFile']:
                f.write(str(self.config['SubmitFile']['executable']))
            else:
                f.write(str(os.path.basename(self.get_glidein_script())))

            mode = os.fstat(f.fileno()).st_mode
            mode |= 0o111
            os.fchmod(f.fileno(), mode & 0o7777)

    def make_submit_file(self, filename, env_wrapper, state, group_jobs, cluster_config,
                         presigned_put_url=None, presigned_get_url=None):
        """
        Creating HTCondor submit file

        Args:
            filename: name of HTCondor submit file
            env_wrapper: name of wrapper script
            state: what resource requirements a given glidein has
        """
        with open(filename, 'w') as f:
            if "custom_header" in self.config["SubmitFile"]:
                self.write_line(f, self.config["SubmitFile"]["custom_header"])

            if ('Mode' in self.config and 'debug' in self.config['Mode']
               and self.config["Mode"]["debug"]):
                outdir = os.path.join(os.getcwd(),'out')
                if not os.path.isdir(outdir):
                    os.mkdir(outdir)
                self.write_line(f, "output = %s/$(Cluster).out"%outdir)
                self.write_line(f, "error = %s/$(Cluster).out"%outdir)
            else:
                self.write_line(f, "output = /dev/null")
                self.write_line(f, "error = /dev/null")
            if 'log' in self.config['SubmitFile']:
                self.write_line(f, "log = "+self.config['SubmitFile']['log'])
            else:
                self.write_line(f, "log = log")
            self.write_line(f, "notification = never")
            self.write_line(f, "should_transfer_files = YES")
            self.write_line(f, "when_to_transfer_output = ON_EXIT")
            self.write_line(f, "want_graceful_removal = True")
            self.write_line(f, "executable = %s" % env_wrapper)
            self.write_line(f, "+TransferOutput=\"\"")

            # get input files
            infiles = []
            glidein_script = self.get_glidein_script()
            if not os.path.isfile(glidein_script):
                raise Exception("no glidein_script provided")
            infiles.append(glidein_script)
            osarch_script = os.path.join(os.path.dirname(glidein_script),'os_arch.sh')
            if not os.path.isfile(osarch_script):
                raise Exception("os_arch.sh not found")
            infiles.append(osarch_script)
            log_shipper_script = os.path.join(os.path.dirname(glidein_script),'log_shipper.sh')
            if not os.path.isfile(log_shipper_script):
                raise Exception("log_shipper_script.sh not found")
            infiles.append(log_shipper_script)
            if "tarball" in self.config["Glidein"]:
                if not os.path.isfile(self.config["Glidein"]["tarball"]):
                    raise Exception("provided tarball does not exist")
                infiles.append(self.config["Glidein"]["tarball"])
            # Adding StartD Cron Scripts
            if self.config.get("StartdChecks", {}).get("enable_startd_checks", True):
                startd_cron_scripts_dir = os.path.join(os.path.dirname(glidein_script),
                                                       'startd_cron_scripts')
                if not os.path.isdir(startd_cron_scripts_dir):
                    raise Exception("StartD cron scripts directory not found: "
                                    "{}".format(startd_cron_scripts_dir))
                for script in self.startd_cron_scripts:
                    script_path = os.path.join(startd_cron_scripts_dir, script)
                    if not os.path.isfile(script_path):
                        raise Exception("Stard cron script not found: {}".format(script))
                    infiles.append(os.path.join(startd_cron_scripts_dir, script))
            self.write_line(f, "transfer_input_files = %s"%(','.join(infiles)))

            if "custom_middle" in self.config["SubmitFile"]:
                self.write_line(f, self.config["SubmitFile"]["custom_middle"])

            if cluster_config['whole_node']:
                num_cpus = int(cluster_config['whole_node_cpus'])
                mem = int(cluster_config['whole_node_memory'])
                disk = int(cluster_config['whole_node_disk'])*1000
                if 'whole_node_gpus' in cluster_config:
                    num_gpus = int(cluster_config['whole_node_gpus'])
                else:
                    num_gpus = 0
                self.write_line(f, 'request_cpus=%d' % num_cpus)
                self.write_line(f, 'request_memory=%d' % mem)
                self.write_line(f, 'request_disk=%d' % disk)
                if state["gpus"] != 0 and num_gpus:
                    self.write_line(f, 'request_gpus=%d' % num_gpus)
            else:
                if state["cpus"] != 0:
                    self.write_line(f, 'request_cpus=%d' % state["cpus"])
                if state["memory"] != 0:
                    mem_safety_margin = 1.1*self.get_resource_limit_scale("mem_safety_scale")
                    self.write_line(f, 'request_memory=%d' % int(state["memory"]*mem_safety_margin))
                if state["disk"] != 0:
                    self.write_line(f, 'request_disk=%d' % int(state["disk"]*1024*1.1))
                if state["gpus"] != 0:
                    self.write_line(f, 'request_gpus=%d' % int(state["gpus"]))

            # Creating environment variables
            environment_variables = ''
            if presigned_put_url is not None and presigned_get_url is not None:
                environment_variables += ('PRESIGNED_PUT_URL={} '
                                          'PRESIGNED_GET_URL={} ').format(presigned_put_url,
                                                                          presigned_get_url)
            if not self.config.get("StartdChecks", {}).get("enable_startd_checks", True):
                environment_variables += 'DISABLE_STARTD_CHECKS=1'
            if environment_variables != '':
                self.write_line(f, 'environment = "%s"' % environment_variables)

            if "custom_footer" in self.config["SubmitFile"]:
                self.write_line(f, self.config["SubmitFile"]["custom_footer"])
            if group_jobs:
                self.write_line(f, 'queue %d' % state["count"])
            else:
                self.write_line(f, 'queue')

    def submit(self, state, partition="Cluster"):
        """
        Writing submit file and submitting a HTCondor job

        Args:
            state: what resource requirements a given glidein has
        """
        submit_filename = 'submit.condor'
        if 'filename' in self.config["SubmitFile"]:
            submit_filename = self.config["SubmitFile"]["filename"]
        env_filename = 'env_wrapper.sh'
        if 'env_wrapper_name' in self.config['SubmitFile']:
            env_filename = self.config["SubmitFile"]["env_wrapper_name"]

        cluster_config = self.config[partition]
        group_jobs = ("group_jobs" in cluster_config and
                      cluster_config["group_jobs"] and
                      "count" in state)
        self.make_env_wrapper(env_filename, cluster_config)
        num_submits = 1 if group_jobs else state["count"] if "count" in state else 1
        for i in range(num_submits):
            if self.config.get('StartdLogging', {}).get('send_startd_logs', False) is True:
                startd_logfile_name = '{}_{}.tar.gz'.format(self.config['Glidein']['site'],
                                                            uuid.uuid4())
                presigned_put_url = get_presigned_put_url(startd_logfile_name, self.config,
                                                          self.secrets)
                presigned_get_url = get_presigned_get_url(startd_logfile_name, self.config,
                                                          self.secrets)
                self.make_submit_file(submit_filename,
                                      env_filename,
                                      state,
                                      group_jobs,
                                      cluster_config,
                                      presigned_put_url,
                                      presigned_get_url)
            else:
                self.make_submit_file(submit_filename,
                                      env_filename,
                                      state,
                                      group_jobs,
                                      cluster_config)
            cmd = cluster_config["submit_command"] + " " + submit_filename
            subprocess.check_call(cmd, shell=True)



#!/usr/bin/env python
from __future__ import absolute_import, division, print_function

import os
import sys
import time
import platform
import subprocess
import logging
import threading
from functools import partial
from optparse import OptionParser
import urllib
import urllib2
import tempfile
import shutil

from util import json_encode
from client_util import get_state

logger = logging.getLogger('client')


def ssh_write(host,state):
    d = tempfile.mkdtemp()
    try:
        filename = os.path.join(d,'glidein_state')
        with open(filename,'w') as f:
            f.write(json_encode(state))
        cmd = ['scp',filename,host+':~/glidein_state']
        if subprocess.call(cmd):
            raise Exception('error in ssh copy of state')
    finally:
        shutil.rmtree(d)

def main():
    parser = OptionParser()
    parser.add_option('--address',type='string',default='http://bosco.icecube.wisc.edu:9070',
                      help='Address to connect to (default: http://bosco.icecube.wisc.edu:9070)')
    parser.add_option('--debug',action='store_true',default=False,
                      help='Enable debug logging')
    parser.add_option('--ssh-host',dest='ssh_host',type='string',
                      default='',help='ssh host')
    (options,args) = parser.parse_args()

    if options.debug:
        logging.basicConfig(level=logging.DEBUG)
    else:
        logging.basicConfig(level=logging.INFO)

    state = get_state(options.address)
    if state:
        try:
            ssh_write(options.ssh_host,state)
        except Exception:
            logger.warn('error',exc_info=True)
        else:
            logger.info('done')

if __name__ == '__main__':
    main()
